@@ -43,6 +43,7 @@ def main():
                         default="",
                         help="Path to chrome debugger script which can validate " +
                         "Semantic Scholar Search params (optional)")
+    parser.add_argument("--debug", action="store_true", help="Run in debug mode")
     parser.add_argument("--verbosity", "-v", type=str, default="info",
                         help="Verbosity level. One of [error, info, debug]")
     parser.add_argument("--version", action="store_true",
@@ -64,6 +65,7 @@ def main():
               "config_dir": args.config_dir,
               "batch_size": args.batch_size,
               "chrome_debugger_path": args.chrome_debugger_path,
+              "debug": args.debug,
               "verbosity": args.verbosity,
               "threaded": args.threaded}
     service = RefMan(**kwargs)
